import React from 'react';
import { Dialog, ReactWidget } from '@jupyterlab/apputils';
import { IDatasource } from './api/datasource';
import { Widget } from '@lumino/widgets';

interface IDatasourceItem {
  address: string;
  username: string;
  password: string;
  databasename: string;
  type: 'mysql' | 'hive';
}

interface IProps {
  value: Partial<IDatasourceItem>;
  error: keyof IDatasourceItem | null;
  onChange: (v: Partial<IDatasourceItem>) => void;
}

const DatasourceForm: React.FunctionComponent<IProps> = ({
  value,
  error,
  onChange
}) => {
  return (
    <form className="jldbq-form">
      <label className={error === 'type' ? 'jldbq-form-error' : ''}>
        <span className="jldbq-form-label">数据源类型:</span>
        <div className="jldbq-form-switcher">
          <button
            type="button"
            className={
              'jldbq-btn ' + (value.type === 'mysql' ? 'jldbq-btn-current' : '')
            }
            onClick={() => onChange({ type: 'mysql' })}
          >
            MySQL
          </button>
          <button
            type="button"
            className={
              'jldbq-btn ' + (value.type === 'hive' ? 'jldbq-btn-current' : '')
            }
            onClick={() => onChange({ type: 'hive' })}
          >
            Hive
          </button>
        </div>
      </label>
      <label className={error === 'address' ? 'jldbq-form-error' : ''}>
        <span className="jldbq-form-label">数据源地址:</span>
        <input
          className="jldbq-form-input"
          placeholder="请输入数据源地址"
          value={value.address || ''}
          onChange={evt => onChange({ address: evt.target.value })}
        />
      </label>
      <label className={error === 'username' ? 'jldbq-form-error' : ''}>
        <span className="jldbq-form-label">用户名:</span>
        <input
          className="jldbq-form-input"
          placeholder="请输入用户名"
          value={value.username || ''}
          onChange={evt => onChange({ username: evt.target.value })}
        />
      </label>
      <label className={error === 'password' ? 'jldbq-form-error' : ''}>
        <span className="jldbq-form-label">密码:</span>
        <input
          className="jldbq-form-input"
          type="password"
          placeholder="请输入密码"
          value={value.password || ''}
          onChange={evt => onChange({ password: evt.target.value })}
        />
      </label>
      <label className={error === 'databasename' ? 'jldbq-form-error' : ''}>
        <span className="jldbq-form-label">数据库名称:</span>
        <input
          className="jldbq-form-input"
          placeholder="请输入数据库名称"
          value={value.databasename || ''}
          onChange={evt => onChange({ databasename: evt.target.value })}
        />
      </label>
    </form>
  );
};

export class DatasourceFormDialogBody
  extends ReactWidget
  implements Dialog.IBodyWidget<IDatasource> {
  constructor(manager: string, options?: Widget.IOptions) {
    super(options);
    this._manager = manager;
  }
  render(): JSX.Element {
    return (
      <DatasourceForm
        value={this._datasourceItem}
        error={this._error}
        onChange={v => {
          this._datasourceItem = { ...this._datasourceItem, ...v };
          this._error = null;
          this.update();
        }}
      />
    );
  }

  getValue(): IDatasource {
    const {
      address,
      username: user,
      password,
      databasename,
      type: source
    } = this._datasourceItem;
    if (!source) {
      this._setError('type');
    }
    if (!address || !validateIpAndPort(address)) {
      this._setError('address');
    }
    const [host, portStr] = address.split(':');
    const port = portStr ? +portStr : 3306;
    if (!user) {
      this._setError('username');
    }
    if (!password) {
      this._setError('password');
    }
    if (!databasename) {
      this._setError('databasename');
    }
    return {
      host,
      user,
      port,
      password,
      charset: 'utf8',
      source,
      databasename,
      sourcename: databasename,
      manager: this._manager
    };
  }

  private _setError(error: keyof IDatasourceItem): never {
    this._error = error;
    this.update();
    throw new Error(error);
  }

  private _datasourceItem: Partial<IDatasourceItem> = {};
  private _error: IProps['error'] = null;
  private _manager: string;
}

function validateIpAndPort(input: string): boolean {
  const parts = input.split(':');
  if (parts.length > 2) {
    return false;
  }
  const ip = parts[0].split('.');
  if (
    ip.length !== 4 ||
    !ip.every(function (segment) {
      return validateNum(segment, 0, 255);
    })
  ) {
    return false;
  }
  const port = parts[1];
  if (port && !validateNum(port, 1, 65535)) {
    return false;
  }
  return true;
}

function validateNum(input: string, min: number, max: number): boolean {
  const num = +input;
  return num >= min && num <= max && input === num.toString();
}
