import useSWR from 'swr';
import config from './config';

export interface IDatasource {
  host: string;
  user: string;
  port: number;
  password: string;
  charset: 'utf8';
  manager: string;
  source: 'mysql' | 'hive';
  databasename: string;
  sourcename: string;
}

interface IDatasourceResponse {
  datasources?: IDatasource[];
  loading: boolean;
  error?: any;
  refresh: () => void;
}

export type ITableData = Record<string, Record<string, any>>;

export const useDatasourceList = (manager: string): IDatasourceResponse => {
  const endpoint = config.endpoint;
  const item = encodeURIComponent(manager);
  const url = `${endpoint}/managelist/item=${item}`;
  const { data, error, mutate } = useSWR(url, (...args) =>
    fetch(...args).then(res => res.json())
  );
  return {
    datasources: data,
    loading: !error && !data,
    error,
    refresh: mutate
  };
};

export const setDatasource = async (
  datasource: IDatasource
): Promise<string> => {
  const endpoint = config.endpoint;
  const url = `${endpoint}/manage`;
  try {
    const resp = await fetch(url, {
      method: 'post',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify(datasource)
    });
    if (!resp.ok) {
      return 'error';
    }
    const data = await resp.text();
    return data;
  } catch (e) {
    console.error(e);
    return 'error';
  }
};

export const fetchTables = async (
  ds: IDatasource
): Promise<'error' | string[]> => {
  const endpoint = config.endpoint;
  const item = encodeURIComponent(
    `${ds.manager};${ds.sourcename};${ds.databasename}`
  );
  const url = `${endpoint}/show/${ds.source}/tables/item=${item}`;
  try {
    const resp = await fetch(url);
    if (!resp.ok) {
      return 'error';
    }
    const data = (await resp.json()) as string[];
    return data;
  } catch (e) {
    console.error(e);
    return 'error';
  }
};

export const fetchTableContent = async (
  ds: IDatasource,
  t: string
): Promise<ITableData | 'error'> => {
  const endpoint = config.endpoint;
  const item = encodeURIComponent(
    `${ds.manager};${ds.sourcename};${ds.databasename};${t}`
  );
  const url = `${endpoint}/show/${ds.source}/results/item=${item}`;
  try {
    const resp = await fetch(url);
    if (!resp.ok) {
      return 'error';
    }
    const data = (await resp.json()) as ITableData;
    return data;
  } catch (e) {
    console.error(e);
    return 'error';
  }
};
